@@ -27,6 +27,7 @@ import abc
 import netaddr
 import rfc3986
 import six
+from six.moves import range as compat_range
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -156,8 +157,8 @@ class String(ConfigType):
 
     def __repr__(self):
         details = []
-        if self.choices:
-            details.append("choices=%r" % self.choices)
+        if self.choices is not None:
+            details.append("choices={!r}".format(self.choices))
         if self.regex:
             details.append("regex=%r" % self.regex.pattern)
         if details:
@@ -243,88 +244,69 @@ class Boolean(ConfigType):
         return str(value).lower()
 
 
-class Integer(ConfigType):
+class Number(ConfigType):
 
-    """Integer type.
-
-    Converts value to an integer optionally doing range checking.
-    If value is whitespace or empty string will return None.
+    """Number class, base for Integer and Float.
 
     :param min: Optional check that value is greater than or equal to min.
-                Mutually exclusive with 'choices'.
     :param max: Optional check that value is less than or equal to max.
-                Mutually exclusive with 'choices'.
     :param type_name: Type name to be used in the sample config file.
-    :param choices: Optional sequence of valid values. Mutually exclusive
-                    with 'min/max'.
+    :param choices: Optional sequence of valid values.
+    :param num_type: the type of number used for casting (i.e int, float)
 
-    .. versionchanged:: 2.4
-       The class now honors zero for *min* and *max* parameters.
-
-    .. versionchanged:: 2.7
-       Added *type_name* parameter.
-
-    .. versionchanged:: 3.2
-       Added *choices* parameter.
+    .. versionadded:: 3.14
     """
 
-    def __init__(self, min=None, max=None, type_name='integer value',
-                 choices=None):
-        super(Integer, self).__init__(type_name=type_name)
-        if choices is not None:
-            if min is not None or max is not None:
-                raise ValueError("'choices' and 'min/max' cannot both be "
-                                 "specified")
-        else:
-            if min is not None and max is not None and max < min:
-                raise ValueError('Max value is less than min value')
+    def __init__(self, num_type, type_name,
+                 min=None, max=None, choices=None):
+        super(Number, self).__init__(type_name=type_name)
+
+        if min is not None and max is not None and max < min:
+            raise ValueError('Max value is less than min value')
+        invalid_choices = [c for c in choices or []
+                           if (min is not None and min > c)
+                           or (max is not None and max < c)]
+        if invalid_choices:
+            raise ValueError("Choices %s are out of bounds [%s..%s]"
+                             % (invalid_choices, min, max))
         self.min = min
         self.max = max
         self.choices = choices
+        self.num_type = num_type
 
     def __call__(self, value):
-        if not isinstance(value, int):
+        if not isinstance(value, self.num_type):
             s = str(value).strip()
             if s == '':
-                value = None
-            else:
-                value = int(value)
+                return None
+            value = self.num_type(value)
 
-        if value is not None:
-            if self.choices is not None:
-                self._check_choices(value)
-            else:
-                self._check_range(value)
-
-        return value
-
-    def _check_choices(self, value):
-        if value in self.choices:
-            return
+        if self.choices is None:
+            if self.min is not None and value < self.min:
+                raise ValueError('Should be greater than or equal to %g' %
+                                 self.min)
+            if self.max is not None and value > self.max:
+                raise ValueError('Should be less than or equal to %g' %
+                                 self.max)
         else:
-            raise ValueError('Valid values are %r, but found %d' % (
-                             self.choices, value))
-
-    def _check_range(self, value):
-        if self.min is not None and value < self.min:
-            raise ValueError('Should be greater than or equal to %d' %
-                             self.min)
-        if self.max is not None and value > self.max:
-            raise ValueError('Should be less than or equal to %d' % self.max)
+            if value not in self.choices:
+                raise ValueError('Valid values are %r, but found %g' % (
+                                 self.choices, value))
+        return value
 
     def __repr__(self):
         props = []
         if self.choices is not None:
-            props.append("choices=%r" % (self.choices,))
+            props.append("choices={!r}".format(self.choices))
         else:
             if self.min is not None:
-                props.append('min=%d' % self.min)
+                props.append('min=%g' % self.min)
             if self.max is not None:
-                props.append('max=%d' % self.max)
+                props.append('max=%g' % self.max)
 
         if props:
-            return 'Integer(%s)' % ', '.join(props)
-        return 'Integer'
+            return self.__class__.__name__ + '(%s)' % ', '.join(props)
+        return self.__class__.__name__
 
     def __eq__(self, other):
         return (
@@ -340,34 +322,89 @@ class Integer(ConfigType):
         return str(value)
 
 
-class Float(ConfigType):
+class Integer(Number):
+
+    """Integer type.
+
+    Converts value to an integer optionally doing range checking.
+    If value is whitespace or empty string will return None.
+
+    :param min: Optional check that value is greater than or equal to min.
+    :param max: Optional check that value is less than or equal to max.
+    :param type_name: Type name to be used in the sample config file.
+    :param choices: Optional sequence of valid values.
+
+    .. versionchanged:: 2.4
+       The class now honors zero for *min* and *max* parameters.
+
+    .. versionchanged:: 2.7
+       Added *type_name* parameter.
+
+    .. versionchanged:: 3.2
+       Added *choices* parameter.
+
+    .. versionchanged:: 3.16
+       *choices* is no longer mutually exclusive with *min*/*max*. If those are
+       supplied, all choices are verified to be within the range.
+    """
+
+    def __init__(self, min=None, max=None, type_name='integer value',
+                 choices=None):
+        super(Integer, self).__init__(int, type_name, min=min, max=max,
+                                      choices=choices)
+
+
+class Float(Number):
 
     """Float type.
 
     :param type_name: Type name to be used in the sample config file.
+    :param min: Optional check that value is greater than or equal to min.
+    :param max: Optional check that value is less than or equal to max.
 
     .. versionchanged:: 2.7
 
        Added *type_name* parameter.
+
+    .. versionchanged:: 3.14
+
+       Added *min* and *max* parameters. If *choices* are also supplied, they
+       must be within the range.
     """
 
-    def __init__(self, type_name='floating point value'):
-        super(Float, self).__init__(type_name=type_name)
+    def __init__(self, min=None, max=None, type_name='floating point value'):
+        super(Float, self).__init__(float, type_name, min=min, max=max)
 
-    def __call__(self, value):
-        if isinstance(value, float):
-            return value
 
-        return float(value)
+class Port(Integer):
 
-    def __repr__(self):
-        return 'Float'
+    """Port type
 
-    def __eq__(self, other):
-        return self.__class__ == other.__class__
+    Represents a L4 Port.
 
-    def _formatter(self, value):
-        return str(value)
+    :param type_name: Type name to be used in the sample config file.
+    :param choices: Optional sequence of valid values.
+    :param min: Optional check that value is greater than or equal to min.
+    :param max: Optional check that value is less than or equal to max.
+
+    .. versionadded:: 3.16
+    """
+
+    PORT_MIN = 0
+    PORT_MAX = 65535
+
+    def __init__(self, min=None, max=None, type_name='port', choices=None):
+        min = self.PORT_MIN if min is None else min
+        max = self.PORT_MAX if max is None else max
+        if min < self.PORT_MIN:
+            raise ValueError('Min value cannot be less than %(min)d',
+                             {'min': self.PORT_MIN})
+        if max > self.PORT_MAX:
+            raise ValueError('Max value cannot be more than %(max)d',
+                             {'max': self.PORT_MAX})
+
+        super(Port, self).__init__(min=min, max=max, type_name=type_name,
+                                   choices=choices)
 
 
 class List(ConfigType):
@@ -454,6 +491,62 @@ class List(ConfigType):
         return ','.join(value)
 
 
+class Range(ConfigType):
+
+    """Range type.
+
+    Represents a range of integers. A range is identified by an integer both
+    sides of a '-' character. Negatives are allowed. A single number is also a
+    valid range.
+
+    :param min: Optional check that lower bound is greater than or equal to
+                min.
+    :param max: Optional check that upper bound is less than or equal to max.
+    :param inclusive: True if the right bound is to be included in the range.
+    :param type_name: Type name to be used in the sample config file.
+
+    .. versionadded:: 3.16
+    """
+
+    def __init__(self, min=None, max=None, inclusive=True,
+                 type_name='range value'):
+        super(Range, self).__init__(type_name)
+        self.min = min
+        self.max = max
+        self.inclusive = inclusive
+
+    def __call__(self, value):
+        value = str(value)
+        num = "0|-?[1-9][0-9]*"
+        m = re.match("^(%s)(?:-(%s))?$" % (num, num), value)
+        if not m:
+            raise ValueError('Invalid Range: %s' % value)
+        left = int(m.group(1))
+        right = int(left if m.group(2) is None else m.group(2))
+
+        if left < right:
+            left = Integer(min=self.min)(left)
+            right = Integer(max=self.max)(right)
+            step = 1
+        else:
+            left = Integer(max=self.max)(left)
+            right = Integer(min=self.min)(right)
+            step = -1
+        if self.inclusive:
+            right += step
+        return compat_range(left, right, step)
+
+    def __eq__(self, other):
+        return (
+            (self.__class__ == other.__class__) and
+            (self.min == other.min) and
+            (self.max == other.max)
+        )
+
+    def _formatter(self, value):
+        return value
+
+
 class Dict(ConfigType):
 
     """Dictionary type.
@@ -664,17 +757,39 @@ class URI(ConfigType):
 
     Represents URI. Value will be validated as RFC 3986.
 
+    :param max_length: Optional integer. If a positive value is specified,
+                       a maximum length of an option value must be less than
+                       or equal to this parameter. Otherwise no length check
+                       will be done.
+    :param schemes: List of valid schemes.
     :param type_name: Type name to be used in the sample config file.
 
+    .. versionchanged:: 3.14
+       Added *max_length* parameter.
+    .. versionchanged:: 3.18
+       Added *schemes* parameter.
     """
 
-    def __init__(self, type_name='uri value'):
+    def __init__(self, max_length=None, schemes=None, type_name='uri value'):
         super(URI, self).__init__(type_name=type_name)
+        self.max_length = max_length
+        self.schemes = schemes
 
     def __call__(self, value):
         if not rfc3986.is_valid_uri(value, require_scheme=True,
                                     require_authority=True):
             raise ValueError('invalid URI: %r' % value)
+
+        if self.max_length is not None and len(value) > self.max_length:
+            raise ValueError("Value '%s' exceeds maximum length %d" %
+                             (value, self.max_length))
+
+        if self.schemes:
+            scheme = rfc3986.uri_reference(value).scheme
+            if scheme not in self.schemes:
+                raise ValueError("URI scheme '%s' not in %s" %
+                                 (scheme, self.schemes))
+
         self.value = value
         return value
 
